"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
exports.__esModule = true;
var passport = require("passport");
var passport_dropbox_oauth2_1 = require("passport-dropbox-oauth2");
var url = require("url");
/**
 * Sets up Dropbox Oauth authentication on your express application
 *
 * 1. First passport is set up using the `appKey` and `appSecret`.
 * 2. Then when the user visits `authStartPath`, they are redirected to
 * Dropbox's oauth server.
 * 3. The user then logs in on Dropbox's server
 * 4. The Dropbox server responds with the authentication code to `callbackURL`
 * 5. Passport makes a request for the Token and Profile to the Dropbox API
 * using `appKey`, `appSecret` and the `code` it just got back.
 * 6. If the token and profile is fetched successful, we redirect to
 * `successRedirect`, otherwise we redirect to `failureRedirect`
 *
 * @param appKey This is the app key/clientID you get from creating your Dropbox application
 * (https://docs.gravityforms.com/creating-a-custom-dropbox-app/) (ex: 3u23809sd90239)
 * @param appSecret This is the app secret/client secret you get from creating your Dropbox application
 * (ex: 3u23809sd90239)
 * @param onOauthSuccess The callback for when the oauth is successful and we have the token. Usually you
 * would link the token to a user using the `userID`. This function has to return a promise that resolves
 * when you finish processing. It must resolve the promise with a user object
 *
 * @param expressApp - The express app that you want to create routes on
 * @param authStartPath - (optional) The path the user should go to to start authentication (default: /auth/dropbox)
 * @param callbackURL - The full url that the Dropbox Oauth server will respond to including the domain
 * (ex: http://yourdomain.com/auth/dropbox/response). The app will extract the path and automatically create
 * a handler on it to read the response
 * @param successRedirect - Once the oauth flow completes successfully where should the user be redirected to
 * (ex: /appHome)
 * @param failureRedirect - If the oauth flow fails, where should the user be redirected to
 * (ex: /login)
 */
exports["default"] = (function (_a) {
    var appKey = _a.appKey, appSecret = _a.appSecret, onOauthSuccess = _a.onOauthSuccess, expressApp = _a.expressApp, _b = _a.authStartPath, authStartPath = _b === void 0 ? "/auth/dropbox" : _b, callbackURL = _a.callbackURL, successRedirect = _a.successRedirect, failureRedirect = _a.failureRedirect;
    expressApp.use(passport.initialize());
    // set up passport authentication
    passport.use(new passport_dropbox_oauth2_1.Strategy({
        apiVersion: "2",
        clientID: appKey,
        clientSecret: appSecret,
        callbackURL: callbackURL
    }, function (accessToken, refreshToken, profile, done) { return __awaiter(void 0, void 0, void 0, function () {
        var user, mutatedUser;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    user = {
                        userId: profile.id,
                        email: profile.emails[0].value,
                        name: {
                            givenName: profile.name.givenName,
                            familyName: profile.name.familyName,
                            displayName: profile.displayName
                        },
                        // any other user details
                        profile: profile
                    };
                    return [4 /*yield*/, onOauthSuccess({
                            accessToken: accessToken,
                            refreshToken: refreshToken,
                            user: user
                        })];
                case 1:
                    mutatedUser = _a.sent();
                    done(null, mutatedUser);
                    return [2 /*return*/];
            }
        });
    }); }));
    expressApp.get(authStartPath, passport.authenticate("dropbox-oauth2"));
    // ex: http://localhost:8888/dropbox-callback becomes /dropbox-callback
    var callbackPath = url.parse(callbackURL).path;
    expressApp.get(callbackPath, passport.authenticate("dropbox-oauth2", {
        failureRedirect: failureRedirect,
        successRedirect: successRedirect,
        session: false
    }));
});
